from rdflib import URIRef, Namespace
from rdflib.namespace import RDF
from pyproms.prov_agent import ProvAgent


# TODO: convert this to a subclass of type Agent, not directly from RdfClass
class PromsReportingSystem(ProvAgent):
    """
    Creates a PROMS-O ReportingAystem instance
    """
    def __init__(self,
                 label,
                 uri=None,
                 comment=None,
                 actedOnBehalfOf=None):

        ProvAgent.__init__(self,
                           label,
                           uri=uri,
                           comment=comment,
                           actedOnBehalfOf=actedOnBehalfOf
                           )

    def make_graph(self):
        """
        Specialises ProvAgent.make_graph()

        :return: an rdflib Graph object
        """
        ProvAgent.make_graph(self)

        PROMS = Namespace('http://promsns.org/def/proms#')
        self.g.bind('proms', PROMS)
        PROV = Namespace('http://www.w3.org/ns/prov#')
        self.g.bind('prov', PROV)

        self.g.remove((
            URIRef(self.uri),
            RDF.type,
            PROV.Agent))
        self.g.add((
            URIRef(self.uri),
            RDF.type,
            PROMS.ReportingSystem))



from rdflib import URIRef, Namespace
from rdflib.namespace import RDF, OWL
from pyproms.owlclass import OwlClass


# TODO: split Agent into People and Software Agents for more in-depth handling later
class ProvAgent(OwlClass):
    """
    Creates a PROV-O Agent instance
    """
    def __init__(
            self,
            label,
            uri=None,
            comment=None,
            actedOnBehalfOf=None):

        OwlClass.__init__(self, label, uri, comment)

        if actedOnBehalfOf:
            self.set_actedOnBehalfOf(actedOnBehalfOf)
        else:
            self.actedOnBehalfOf = None

    def set_actedOnBehalfOf(self, actedOnBehalfOf):
        if type(actedOnBehalfOf) is ProvAgent:
            self.actedOnBehalfOf = actedOnBehalfOf
        else:
            raise TypeError('wasAttributedTo must be an Agent, not a %s' % type(actedOnBehalfOf))

    def make_graph(self):
        """
        Specialises RdfClass.make_graph()

        :return: an rdflib Graph object
        """
        OwlClass.make_graph(self)

        PROV = Namespace('http://www.w3.org/ns/prov#')
        self.g.bind('prov', PROV)

        self.g.remove((
            URIRef(self.uri),
            RDF.type,
            OWL.Class))
        self.g.add((
            URIRef(self.uri),
            RDF.type,
            PROV.Agent))

        if self.actedOnBehalfOf:
            self.g = self.g + self.actedOnBehalfOf.get_graph()
            self.g.add((URIRef(self.uri),
                        PROV.actedOnBehalfOf,
                        URIRef(self.actedOnBehalfOf.uri)))

    def get_graph(self):
        """
        Generates the RDF graph of this class

        :return: This class's RDF graph
        """
        if not self.g:
            self.make_graph()

        return self.g



import os
import requests
from pyproms.proms_report import PromsReport
from rdflib import Graph, util


class ReportSender:
    """
    A tiny class to send Reports to a PROMS Server instance.
    Basically wraps requests post method
    """

    def post(self, proms_report_lodging_uri, report):
        """
        POSTS an RDF-serialised Report object to a PROMS server instance

        :param proms_report_lodging_uri: the URI of the PROMS server instance's Report lodgement endpoint.
        Typically something like {PROMS_URI}/function/lodge-report.
        :param report: a pyproms Report class object, an rdflib Graph of a Report, a Report file path or a string
        containing RDF of a Report in turtle

        :return: a requests module Response class
        """

        if isinstance(report, PromsReport):
            report_str = report.serialize_graph().decode('utf-8')
        elif isinstance(report, Graph):
            report_str = report.serialize(format='turtle')
        elif isinstance(report, str):
            if os.path.exists(report):
                g = Graph()
                g.parse(report, format=util.guess_format(report))
                report_str = g.serialize(format='turtle')
            else:  # assume it's an RDF string in turtle
                report_str = report
        else:  # don't allow anything else
            raise ValueError(
                'Only PromsReport objects, rdflib  Graph objects, path strings to RDF files or a string of '
                'RDF in turtle format are allowd for \'report\''
            )

        # POST the Report to PROMS
        headers = {'Content-type': 'text/turtle'}
        r = requests.post(proms_report_lodging_uri, data=report_str, headers=headers)

        return r



#!/usr/bin/env python
from setuptools import setup
import io

with io.open('README.md', 'r', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='pyproms',
    packages=['pyproms'],
    version='1.4.0',
    description='PyPROMS is a client library to make provenance documents and submit them to a PROMS Server instance',
    author='Nicholas Car',
    author_email='nicholas.car@csiro.au',
    url='https://github.com/CSIRO-enviro-informatics/pyproms',
    download_url='https://github.com/CSIRO-enviro-informatics/pyproms/archive/v1.4.0.tar.gz',
    license='LICENSE.txt',
    keywords=['rdf', 'prov', 'prov-o', 'proms', 'proms-o', 'provenance'],
    long_description=readme,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    project_urls={
        'Bug Reports': 'https://github.com/CSIRO-enviro-informatics/pyproms/issues',
        'Source': 'https://github.com/CSIRO-enviro-informatics/pyproms/',
    },
    python_requires='>=3.0',
    install_requires=[
        'rdflib >= 4.0.0',
        'requests'
    ],
)

 # use http://peterdowns.com/posts/first-time-with-pypi.html


from datetime import datetime
from rdflib import URIRef, Namespace, Literal
from rdflib.namespace import RDF, OWL
from pyproms.owlclass import OwlClass
from pyproms.proms_reportingsystem import PromsReportingSystem
from pyproms.prov_activity import ProvActivity
import uuid


class PromsReport(OwlClass):
    """
    Creates a PROV-O Report instance
    """
    def __init__(self,
                 label,
                 wasReportedBy,
                 nativeId,
                 reportActivity,
                 generatedAtTime,
                 comment=None):

        OwlClass.__init__(self, label, comment)

        self.uri = 'http://placeholder.org#' + str(uuid.uuid4())
        self.__set_wasReportedBy(wasReportedBy)
        self.__set_nativeId(nativeId)
        self.__set_reportActivity(reportActivity)
        self.__set_generatedAtTime(generatedAtTime)

    def __set_wasReportedBy(self, wasReportedBy):
        if type(wasReportedBy) is PromsReportingSystem:
            self.wasReportedBy = wasReportedBy
        else:
            raise TypeError('wasReportedBy must be a ReportingSystem, not a %s' % type(wasReportedBy))

    def __set_nativeId(self, nativeId):
        if type(nativeId) is str:
            self.nativeId = nativeId
        else:
            raise TypeError('nativeId must be a string (str), not a %s' % type(nativeId))

    def __set_reportActivity(self, reportActivity):
        if type(reportActivity) is ProvActivity:
            self.reportActivity = reportActivity
        else:
            raise TypeError('reportActivity must be an Agent, not a %s' % type(reportActivity))

    def __set_generatedAtTime(self, generatedAtTime):
        if type(generatedAtTime) is datetime:
            self.generatedAtTime = generatedAtTime
        else:
            raise TypeError('endedAtTime must be a datetime.datetime, not a %s' % type(generatedAtTime))

    def make_graph(self):
        """
        Specialises RdfClass.make_graph()

        :return: an rdflib Graph object
        """
        OwlClass.make_graph(self)

        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
        PROV = Namespace('http://www.w3.org/ns/prov#')
        self.g.bind('prov', PROV)
        PROMS = Namespace('http://promsns.org/def/proms#')
        self.g.bind('proms', PROMS)

        self.g.remove((
            URIRef(self.uri),
            RDF.type,
            OWL.Class))
        self.g.add((
            URIRef(self.uri),
            RDF.type,
            PROMS.Report))

        self.g = self.g + self.wasReportedBy.get_graph()

        self.g.add((URIRef(self.uri),
                    PROMS.wasReportedBy,
                    URIRef(self.wasReportedBy.uri)))

        self.g.add((URIRef(self.uri),
                    PROMS.nativeId,
                    Literal(self.nativeId, datatype=XSD.string)))

        self.g = self.g + self.reportActivity.get_graph()

        self.g.add((URIRef(self.uri),
                    PROMS.startingActivity,
                    URIRef(self.reportActivity.uri)))

        self.g.add((URIRef(self.uri),
                    PROMS.endingActivity,
                    URIRef(self.reportActivity.uri)))

        self.g.add((URIRef(self.uri),
                    PROV.generatedAtTime,
                    Literal(self.generatedAtTime.isoformat(), datatype=XSD.dateTime)))



from rdflib import URIRef, Literal, Namespace
from rdflib.namespace import RDF, OWL
import datetime
from pyproms.owlclass import OwlClass
from pyproms.prov_agent import ProvAgent


# TODO: allow Entity - Entity relationships
class ProvEntity(OwlClass):
    """
    Creates a PROV-O Entity instance
    """
    def __init__(self,
                 label,
                 uri=None,
                 comment=None,
                 wasAttributedTo=None,
                 value=None):

        OwlClass.__init__(self, label, uri, comment)

        if wasAttributedTo:
            self.__set_wasAttributedTo(wasAttributedTo)
        else:
            self.wasAttributedTo = None

        if value:
            self.__set_value(value)
        else:
            self.value = None

    def __set_wasAttributedTo(self, wasAttributedTo):
        if type(wasAttributedTo) is ProvAgent:
            self.wasAttributedTo = wasAttributedTo
        else:
            raise TypeError('wasAttributedTo must be an Agent, not a %s' % type(wasAttributedTo))

    def __set_value(self, value):
        if (type(value) is str or
                type(value) is float or
                type(value) is int or
                type(value) is datetime.datetime):
                    self.value = value
        else:
            raise TypeError('Entity \'value\' must be a string, int, float or datetime, not a %s' % type(value))

    def make_graph(self):
        """
        Specialises RdfClass.make_graph()

        :return: an rdflib Graph object
        """
        OwlClass.make_graph(self)

        PROV = Namespace('http://www.w3.org/ns/prov#')
        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')

        self.g.remove((
            URIRef(self.uri),
            RDF.type,
            OWL.Class))
        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROV.Entity))

        if self.wasAttributedTo:
            self.g = self.g + self.wasAttributedTo.get_graph()
            self.g.add((
                URIRef(self.uri),
                PROV.wasAttributedTo,
                URIRef(self.wasAttributedTo.uri)))

        if self.value:
            if type(self.value) is datetime.datetime:
                self.g.add((URIRef(self.uri),
                            PROV.value,
                            Literal(self.value, datatype=XSD.dateTime)))
            elif type(self.value) is int:
                self.g.add((URIRef(self.uri),
                            PROV.value,
                            Literal(self.value, datatype=XSD.integer)))
            elif type(self.value) is float:
                self.g.add((URIRef(self.uri),
                            PROV.value,
                            Literal(self.value, datatype=XSD.float)))
            else:  # string
                self.g.add((URIRef(self.uri),
                            PROV.value,
                            Literal(self.value, datatype=XSD.string)))

    def get_graph(self):
        """
        Generates the RDF graph of this class

        :return: This class's RDF graph
        """
        if not self.g:
            self.make_graph()

        return self.g
